@@ -74,7 +74,15 @@ if pwndbg.dbg.is_gdblib_available():
 class Command:
     """Generic command wrapper"""
 
-    builtin_override_whitelist: Set[str] = {"up", "down", "search", "pwd", "start", "ignore"}
+    builtin_override_whitelist: Set[str] = {
+        "up",
+        "down",
+        "search",
+        "pwd",
+        "start",
+        "starti",
+        "ignore",
+    }
     history: Dict[int, str] = {}
 
     def __init__(
@@ -199,10 +207,10 @@ def fix(
     """Fix a single command-line argument coming from the CLI.
 
     Arguments:
-        arg(str): Original string representation (e.g. '0', '$rax', '$rax+44')
-        sloppy(bool): If ``arg`` cannot be evaluated, return ``arg``. (default: False)
-        quiet(bool): If an error occurs, suppress it. (default: True)
-        reraise(bool): If an error occurs, raise the exception. (default: False)
+        arg: Original string representation (e.g. '0', '$rax', '$rax+44')
+        sloppy: If ``arg`` cannot be evaluated, return ``arg``. (default: False)
+        quiet: If an error occurs, suppress it. (default: True)
+        reraise: If an error occurs, raise the exception. (default: False)
 
     Returns:
         Ideally a ``Value`` object.  May return a ``str`` if ``sloppy==True``.
@@ -299,6 +307,10 @@ def fix_int_reraise_arg(arg) -> int:
         )
 
 
+def func_name(function: Callable[P, T]) -> str:
+    return function.__name__.replace("_", "-")
+
+
 def OnlyWhenLocal(function: Callable[P, T]) -> Callable[P, Optional[T]]:
     @functools.wraps(function)
     def _OnlyWhenLocal(*a: P.args, **kw: P.kwargs) -> Optional[T]:
@@ -325,7 +337,7 @@ def OnlyWithFile(function: Callable[P, T]) -> Callable[P, Optional[T]]:
             if pwndbg.aglib.qemu.is_qemu():
                 log.error("Could not determine the target binary on QEMU.")
             else:
-                log.error(f"{function.__name__}: There is no file loaded.")
+                log.error(f"{func_name(function)}: There is no file loaded.")
             return None
 
     return _OnlyWithFile
@@ -338,7 +350,7 @@ def OnlyWhenQemuKernel(function: Callable[P, T]) -> Callable[P, Optional[T]]:
             return function(*a, **kw)
         else:
             log.error(
-                f"{function.__name__}: This command may only be run when debugging the Linux kernel in QEMU."
+                f"{func_name(function)}: This command may only be run when debugging the Linux kernel in QEMU."
             )
             return None
 
@@ -352,39 +364,13 @@ def OnlyWhenUserspace(function: Callable[P, T]) -> Callable[P, Optional[T]]:
             return function(*a, **kw)
         else:
             log.error(
-                f"{function.__name__}: This command may only be run when not debugging a QEMU kernel target."
+                f"{func_name(function)}: This command may only be run when not debugging a QEMU kernel target."
             )
             return None
 
     return _OnlyWhenUserspace
 
 
-def OnlyWithDbg(
-    *dbg_names: Literal["lldb", "gdb"],
-) -> Callable[[Callable[P, T]], Callable[P, Optional[T]]]:
-    """Decorates function to work only with the specified debugger."""
-
-    def decorator(function: Callable[P, T]) -> Callable[P, Optional[T]]:
-        @functools.wraps(function)
-        def _OnlyWithDbg(*a: P.args, **kw: P.kwargs) -> Optional[T]:
-            if pwndbg.dbg.is_gdblib_available():
-                if "gdb" in dbg_names:
-                    return function(*a, **kw)
-            else:
-                if "lldb" in dbg_names:
-                    return function(*a, **kw)
-
-            dbg_str = ", ".join(dbg_names)
-            log.error(
-                f"{function.__name__}: This command may only be run on the {dbg_str} debugger(s)"
-            )
-            return None
-
-        return _OnlyWithDbg
-
-    return decorator
-
-
 def OnlyWithKernelDebugSyms(function: Callable[P, T]) -> Callable[P, Optional[T]]:
     @functools.wraps(function)
     def _OnlyWithKernelDebugSyms(*a: P.args, **kw: P.kwargs) -> Optional[T]:
@@ -392,7 +378,7 @@ def OnlyWithKernelDebugSyms(function: Callable[P, T]) -> Callable[P, Optional[T]
             return function(*a, **kw)
         else:
             log.error(
-                f"{function.__name__}: This command may only be run when debugging a Linux kernel with debug symbols."
+                f"{func_name(function)}: This command may only be run when debugging a Linux kernel with debug symbols."
             )
             return None
 
@@ -405,7 +391,9 @@ def OnlyWhenPagingEnabled(function: Callable[P, T]) -> Callable[P, Optional[T]]:
         if pwndbg.aglib.kernel.paging_enabled():
             return function(*a, **kw)
         else:
-            log.error(f"{function.__name__}: This command may only be run when paging is enabled.")
+            log.error(
+                f"{func_name(function)}: This command may only be run when paging is enabled."
+            )
             return None
 
     return _OnlyWhenPagingEnabled
@@ -418,7 +406,7 @@ def OnlyWhenRunning(function: Callable[P, T]) -> Callable[P, Optional[T]]:
         if pwndbg.aglib.proc.alive:
             return function(*a, **kw)
         else:
-            log.error(f"{function.__name__}: The program is not being run.")
+            log.error(f"{func_name(function)}: The program is not being run.")
             return None
 
     return _OnlyWhenRunning
@@ -432,7 +420,7 @@ def OnlyWithTcache(function: Callable[P, T]) -> Callable[P, Optional[T]]:
             return function(*a, **kw)
         else:
             log.error(
-                f"{function.__name__}: This version of GLIBC was not compiled with tcache support."
+                f"{func_name(function)}: This version of GLIBC was not compiled with tcache support."
             )
             return None
 
@@ -445,7 +433,7 @@ def OnlyWhenHeapIsInitialized(function: Callable[P, T]) -> Callable[P, Optional[
         if pwndbg.aglib.heap.current is not None and pwndbg.aglib.heap.current.is_initialized():
             return function(*a, **kw)
         else:
-            log.error(f"{function.__name__}: Heap is not initialized yet.")
+            log.error(f"{func_name(function)}: Heap is not initialized yet.")
             return None
 
     return _OnlyWhenHeapIsInitialized
@@ -458,7 +446,7 @@ def _try2run_heap_command(function: Callable[P, T], *a: P.args, **kw: P.kwargs)
     try:
         return function(*a, **kw)
     except SymbolUnresolvableError as err:
-        e(f"{function.__name__}: Fail to resolve the symbol: `{err.symbol}`")
+        e(f"{func_name(function)}: Fail to resolve the symbol: `{err.symbol}`")
         if "thread_arena" == err.symbol:
             w(
                 "You are probably debugging a multi-threaded target without debug symbols, so we failed to determine which arena is used by the current thread.\n"
@@ -466,17 +454,17 @@ def _try2run_heap_command(function: Callable[P, T], *a: P.args, **kw: P.kwargs)
             )
         else:
             w(
-                f"You can try to determine the libc symbols addresses manually and set them appropriately. For this, see the `heap_config` command output and set the config for `{err.symbol}`."
+                f"You can try to determine the libc symbols addresses manually and set them appropriately. For this, see the `heap-config` command output and set the config for `{err.symbol}`."
             )
         if pwndbg.config.exception_verbose or pwndbg.config.exception_debugger:
             raise err
 
         pwndbg.exception.inform_verbose_and_debug()
     except Exception as err:
-        e(f"{function.__name__}: An unknown error occurred when running this command.")
+        e(f"{func_name(function)}: An unknown error occurred when running this command.")
         if isinstance(pwndbg.aglib.heap.current, HeuristicHeap):
             w(
-                "Maybe you can try to determine the libc symbols addresses manually, set them appropriately and re-run this command. For this, see the `heap_config` command output and set the `main_arena`, `mp_`, `global_max_fast`, `tcache` and `thread_arena` addresses."
+                "Maybe you can try to determine the libc symbols addresses manually, set them appropriately and re-run this command. For this, see the `heap-config` command output and set the `main_arena`, `mp_`, `global_max_fast`, `tcache` and `thread_arena` addresses."
             )
         else:
             w("You can try `set resolve-heap-via-heuristic force` and re-run this command.\n")
@@ -579,7 +567,9 @@ class _ArgparsedCommand(Command):
     ) -> None:
         self.parser = parser
         if command_name is None:
-            self.parser.prog = function.__name__
+            # Take the command name from the name of the function
+            # which defines it, but replace '_' with '-'.
+            self.parser.prog = function.__name__.replace("_", "-")
         else:
             self.parser.prog = command_name
 
@@ -593,7 +583,7 @@ class _ArgparsedCommand(Command):
         # Type error likely due to https://github.com/python/mypy/issues/6799
         super().__init__(  # type: ignore[misc]
             function,
-            command_name=command_name,
+            command_name=self.parser.prog,
             doc=doc,
             *a,
             **kw,
@@ -610,9 +600,11 @@ class ArgparsedCommand:
     def __init__(
         self,
         parser_or_desc: argparse.ArgumentParser | str,
-        aliases: List[str] = [],
+        category: CommandCategory,
         command_name: str | None = None,
-        category: CommandCategory = CommandCategory.MISC,
+        aliases: List[str] = [],
+        only_debuggers: Set[pwndbg.dbg_mod.DebuggerType] = None,
+        exclude_debuggers: Set[pwndbg.dbg_mod.DebuggerType] = None,
     ) -> None:
         """
         :param parser_or_desc: `argparse.ArgumentParser` instance or `str`
@@ -620,10 +612,15 @@ class ArgparsedCommand:
         if isinstance(parser_or_desc, str):
             self.parser = argparse.ArgumentParser(description=parser_or_desc)
         else:
+            assert isinstance(parser_or_desc, argparse.ArgumentParser)
             self.parser = parser_or_desc
         self.aliases = aliases
+        assert command_name is None or "_" not in command_name
         self._command_name = command_name
+        assert category
         self.category = category
+        self.only_debuggers = only_debuggers
+        self.exclude_debuggers = exclude_debuggers
         # We want to run all integer and otherwise-unspecified arguments
         # through fix() so that GDB parses it.
         for action in self.parser._actions:
@@ -639,6 +636,11 @@ class ArgparsedCommand:
                 action.help += " (default: %(default)s)"
 
     def __call__(self, function: Callable[..., Any]) -> _ArgparsedCommand:
+        if self.only_debuggers is not None and pwndbg.dbg.name() not in self.only_debuggers:
+            return function  # type: ignore[return-value]
+        if self.exclude_debuggers is not None and pwndbg.dbg.name() in self.exclude_debuggers:
+            return function  # type: ignore[return-value]
+
         for alias in self.aliases:
             _ArgparsedCommand(
                 self.parser, function, command_name=alias, is_alias=True, category=self.category
@@ -759,6 +761,7 @@ def load_commands() -> None:
     import pwndbg.commands.knft
     import pwndbg.commands.kversion
     import pwndbg.commands.leakfind
+    import pwndbg.commands.libcinfo
     import pwndbg.commands.linkmap
     import pwndbg.commands.memoize
     import pwndbg.commands.misc
