@@ -6,8 +6,9 @@ import tempfile
 
 import gdb
 
+import pwndbg.aglib.proc
+import pwndbg.aglib.vmmap
 import pwndbg.commands
-import pwndbg.gdblib.vmmap
 from pwndbg.commands import CommandCategory
 
 parser = argparse.ArgumentParser(
@@ -22,17 +23,17 @@ parser.add_argument("argument", nargs="*", type=str, help="Arguments to pass to
 def ropper(argument) -> None:
     with tempfile.NamedTemporaryFile() as corefile:
         # If the process is running, dump a corefile so we get actual addresses.
-        if pwndbg.gdblib.proc.alive:
+        if pwndbg.aglib.proc.alive:
             filename = corefile.name
             gdb.execute(f"gcore {filename}")
         else:
-            filename = pwndbg.gdblib.proc.exe
+            filename = pwndbg.aglib.proc.exe
 
         # Build up the command line to run
         cmd = ["ropper", "--file", filename]
         cmd += argument
 
         try:
-            io = subprocess.call(cmd)
+            subprocess.call(cmd)
         except Exception:
             print("Could not run ropper.  Please ensure it's installed and in $PATH.")
