@@ -4,8 +4,10 @@ import os
 import sys
 from asyncio import CancelledError
 from typing import Any
+from typing import BinaryIO
 from typing import Coroutine
 from typing import List
+from typing import Tuple
 
 import lldb
 
@@ -13,6 +15,7 @@ import pwndbg
 from pwndbg.dbg.lldb import YieldContinue
 from pwndbg.dbg.lldb import YieldSingleStep
 from pwndbg.dbg.lldb.repl.io import IODriver
+from pwndbg.dbg.lldb.repl.io import IODriverPlainText
 
 
 class EventHandler:
@@ -68,6 +71,7 @@ class ProcessDriver:
     listener: lldb.SBListener
     debug: bool
     eh: EventHandler
+    cancellation_requested: bool
 
     def __init__(self, event_handler: EventHandler, debug=False):
         self.io = None
@@ -75,14 +79,46 @@ class ProcessDriver:
         self.listener = None
         self.debug = debug
         self.eh = event_handler
+        self.cancellation_requested = False
 
     def has_process(self) -> bool:
         """
         Whether there's an active process in this driver.
         """
+        return self.process is not None and self.process.GetState() != lldb.eStateConnected
+
+    def has_connection(self) -> bool:
+        """
+        Whether this driver's connected to a target. All drivers that have an
+        active process also must necessarily be connected.
+        """
         return self.process is not None
 
+    def cancel(self) -> None:
+        """
+        Request that a currently ongoing operation be cancelled.
+        """
+        self.cancellation_requested = True
+
+    def _should_cancel(self) -> bool:
+        """
+        Checks whether a cancellation has been requested, and clears cancellation state.
+        """
+        should = self.cancellation_requested
+        self._clear_cancel()
+
+        return should
+
+    def _clear_cancel(self) -> None:
+        """
+        Clears cancellation state.
+        """
+        self.cancellation_requested = False
+
     def interrupt(self) -> None:
+        """
+        Interrupts the currently running process.
+        """
         assert self.has_process(), "called interrupt() on a driver with no process"
         self.process.SendAsyncInterrupt()
 
@@ -93,7 +129,7 @@ class ProcessDriver:
         first_timeout: int = 1,
         only_if_started: bool = False,
         fire_events: bool = True,
-    ) -> lldb.SBEvent | None:
+    ) -> Tuple[bool, lldb.SBEvent | None]:
         """
         Runs the event loop of the process until the next stop event is hit, with
         a configurable timeouts for the first and subsequent timeouts.
@@ -120,7 +156,8 @@ class ProcessDriver:
         # started by a previous action and is running.
         running = not only_if_started
 
-        reason: lldb.SBEvent | None = None
+        last: lldb.SBEvent | None = None
+        expected: bool = False
         while True:
             event = lldb.SBEvent()
             if not self.listener.WaitForEvent(timeout_time, event):
@@ -137,6 +174,7 @@ class ProcessDriver:
                     break
 
                 continue
+            last = event
 
             if self.debug:
                 descr = lldb.SBStream()
@@ -169,7 +207,7 @@ class ProcessDriver:
                         # for the time being. Trigger the stopped event and return.
                         if fire_events:
                             self.eh.suspended()
-                        reason = event
+                        expected = True
                         break
 
                     if new_state == lldb.eStateRunning or new_state == lldb.eStateStepping:
@@ -194,13 +232,12 @@ class ProcessDriver:
 
                         if fire_events:
                             self.eh.exited()
-                        reason = event
                         break
 
         if io_started:
             self.io.stop()
 
-        return reason
+        return expected, last
 
     def cont(self) -> None:
         """
@@ -213,7 +250,7 @@ class ProcessDriver:
         self.process.Continue()
         self._run_until_next_stop()
 
-    def run_lldb_command(self, command: str) -> None:
+    def run_lldb_command(self, command: str, target: BinaryIO) -> None:
         """
         Runs the given LLDB command and ataches I/O if necessary.
         """
@@ -226,23 +263,27 @@ class ProcessDriver:
             # LLDB can give us strings that may fail to encode.
             out = ret.GetOutput().strip()
             if len(out) > 0:
-                sys.stdout.buffer.write(out.encode(sys.stdout.encoding, errors="backslashreplace"))
-                print()
+                target.write(out.encode(sys.stdout.encoding, errors="backslashreplace"))
+                target.write(b"\n")
             out = ret.GetError().strip()
             if len(out) > 0:
-                sys.stdout.buffer.write(out.encode(sys.stdout.encoding, errors="backslashreplace"))
-                print()
+                target.write(out.encode(sys.stdout.encoding, errors="backslashreplace"))
+                target.write(b"\n")
+
+            if self.debug:
+                print(f"[-] ProcessDriver: LLDB Command Status: {ret.GetStatus():#x}")
 
             # Only call _run_until_next_stop() if the command started the process.
+            start_expected = True
             s = ret.GetStatus()
             if s == lldb.eReturnStatusFailed:
-                return
+                start_expected = False
             if s == lldb.eReturnStatusQuit:
-                return
+                start_expected = False
             if s == lldb.eReturnStatusSuccessFinishResult:
-                return
+                start_expected = False
             if s == lldb.eReturnStatusSuccessFinishNoResult:
-                return
+                start_expected = False
 
             # It's important to note that we can't trigger the resumed event
             # now because the process might've already started, and LLDB
@@ -253,7 +294,16 @@ class ProcessDriver:
             #
             # TODO/FIXME: Find a way to trigger the continued event before the process is resumed in LLDB
 
-            self._run_until_next_stop()
+            if not start_expected:
+                # Even if the current process has not been resumed, LLDB might
+                # have posted process events for us to handle. Do so now, but
+                # stop right away if there is nothing for us in the listener.
+                #
+                # This lets us handle things like `detach`, which would otherwise
+                # pass as a command that does not change process state at all.
+                self._run_until_next_stop(first_timeout=0, only_if_started=True)
+            else:
+                self._run_until_next_stop()
 
     def run_coroutine(self, coroutine: Coroutine[Any, Any, None]) -> bool:
         """
@@ -261,8 +311,14 @@ class ProcessDriver:
         process in this driver. Returns `True` if the coroutine ran to completion,
         and `False` if it was cancelled.
         """
+        assert self.has_process(), "called run_coroutine() on a driver with no process"
         exception: Exception | None = None
+        self._clear_cancel()
         while True:
+            if self._should_cancel():
+                # We were requested to cancel the execution controller.
+                exception = CancelledError()
+
             try:
                 if exception is None:
                     step = coroutine.send(None)
@@ -300,14 +356,20 @@ class ProcessDriver:
                     )
                     continue
 
-                self._run_until_next_stop()
+                healthy, event = self._run_until_next_stop()
+                if not healthy:
+                    # The process exited. Cancel the execution controller.
+                    exception = CancelledError()
+                    continue
+
             elif isinstance(step, YieldContinue):
                 # Continue the process and wait for the next stop-like event.
                 self.process.Continue()
-                event = self._run_until_next_stop()
-                assert (
-                    event is not None
-                ), "None should only be returned by _run_until_next_stop unless start timeouts are enabled"
+                healthy, event = self._run_until_next_stop()
+                if not healthy:
+                    # The process exited, Cancel the execution controller.
+                    exception = CancelledError()
+                    continue
 
                 # Check whether this stop event is the one we expect.
                 stop: lldb.SBBreakpoint | lldb.SBWatchpoint = step.target.inner
@@ -364,6 +426,82 @@ class ProcessDriver:
         Launches the process and handles startup events. Always stops on first
         opportunity, and returns immediately after the process has stopped.
 
+        Fires the created() event.
+        """
+        assert not self.has_process(), "called launch() on a driver with a live process"
+
+        error = lldb.SBError()
+
+        # Do the launch, proper. We always stop the target, and let the upper
+        # layers deal with the user wanting the program to not stop at entry by
+        # calling `cont()`.
+        if self.has_connection():
+            # This is a remote launch.
+            #
+            # We ignore the IODriver we were given, and use the plain text
+            # driver, as we can't guarantee that anything else would work.
+            io = IODriverPlainText()
+            stdin, stdout, stderr = io.stdio()
+            self.process.RemoteLaunch(
+                args,
+                env,
+                stdin,
+                stdout,
+                stderr,
+                None,
+                lldb.eLaunchFlagStopAtEntry,
+                True,
+                error,
+            )
+        else:
+            # This is a local launch.
+            self.listener = lldb.SBListener("pwndbg.dbg.lldb.repl.proc.ProcessDriver")
+            assert self.listener.IsValid()
+
+            # We are interested in handling certain target events synchronously, so
+            # set them up here, before LLDB has had any chance to do anything to the
+            # process.
+            self.listener.StartListeningForEventClass(
+                target.GetDebugger(),
+                lldb.SBTarget.GetBroadcasterClassName(),
+                lldb.SBTarget.eBroadcastBitModulesLoaded,
+            )
+            stdin, stdout, stderr = io.stdio()
+            self.process = target.Launch(
+                self.listener,
+                args,
+                env,
+                stdin,
+                stdout,
+                stderr,
+                os.getcwd(),
+                lldb.eLaunchFlagStopAtEntry,
+                True,
+                error,
+            )
+
+            if not error.success:
+                # Undo any initialization Launch() might've done.
+                self.process = None
+                self.listener = None
+
+        if not error.success:
+            return error
+
+        assert self.listener.IsValid()
+        assert self.process.IsValid()
+
+        self.io = io
+        self._run_until_next_stop(fire_events=False)
+        self.eh.created()
+
+        return error
+
+    def attach(self, target: lldb.SBTarget, io: IODriver, info: lldb.SBAttachInfo) -> lldb.SBError:
+        """
+        Attach to a process and handles startup events. Always stops on first
+        opportunity, and returns immediately after the process has stopped.
+
         Fires the created() event.
         """
         stdin, stdout, stderr = io.stdio()
@@ -383,16 +521,9 @@ class ProcessDriver:
         # Do the launch, proper. We always stop the target, and let the upper
         # layers deal with the user wanting the program to not stop at entry by
         # calling `cont()`.
-        self.process = target.Launch(
-            self.listener,
-            args,
-            env,
-            stdin,
-            stdout,
-            stderr,
-            os.getcwd(),
-            lldb.eLaunchFlagStopAtEntry,
-            True,
+        info.SetListener(self.listener)
+        self.process = target.Attach(
+            info,
             error,
         )
 
@@ -414,13 +545,14 @@ class ProcessDriver:
     def connect(self, target: lldb.SBTarget, io: IODriver, url: str, plugin: str) -> lldb.SBError:
         """
         Connects to a remote proces with the given URL using the plugin with the
-        given name, and attaches to the process until LLDB issues a start event
-        to us.
+        given name. This might cause the process to launch in some implementations,
+        or it might require a call to `launch()`, in implementations that
+        require a further call to `SBProcess::RemoteLaunch()`.
 
-        Potentially fires all types of events, as it is not known when LLDB will
-        return control of the process to us.
+        Fires the created() event if a process is automatically attached to
+        or launched when a connection succeeds.
         """
-
+        assert not self.has_connection(), "called connect() on a driver with an active connection"
         stdin, stdout, stderr = io.stdio()
         error = lldb.SBError()
         self.listener = lldb.SBListener("pwndbg.dbg.lldb.repl.proc.ProcessDriver")
@@ -447,10 +579,27 @@ class ProcessDriver:
 
         self.io = io
 
-        # Unlike in `launch()`, it's not guaranteed that the process will not be
-        # running at this point, so we have to attach the I/O and wait until we
-        # get a stop event.
-        self._run_until_next_stop(fire_events=False)
-        self.eh.created()
+        # Unlike in `launch()`, it's not guaranteed that the process is actually
+        # alive, as it might be in the Connected state, which indicates that
+        # the connection was successful, but that we still need to launch it
+        # manually in the remote target.
+        while True:
+            healthy, event = self._run_until_next_stop(
+                with_io=False, fire_events=False, only_if_started=True
+            )
+            if healthy:
+                # The process has startarted. We can fire off the created event
+                # just fine.
+                self.eh.created()
+                break
+
+            if (
+                event is not None
+                and lldb.SBProcess.GetStateFromEvent(event) == lldb.eStateConnected
+            ):
+                # This indicates that on this implementation, connecting isn't
+                # enough to have the process launch or attach, and that we have
+                # to do that manually.
+                break
 
         return error
